/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static int pub_top10(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  putlog(LOG_CMDS, channel, "<<%s>> !%s! top10 %s", nick, hand, text);
  tell_top(channel, channel, text, 1, 10, 0);
  return 1;
}

static int pub_top20(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  putlog(LOG_CMDS, channel, "<<%s>> !%s! top20 %s", nick, hand, text);
  tell_top(channel, channel, text, 11, 20, 0);
  return 1;
}

static int pub_ttop10(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  putlog(LOG_CMDS, channel, "<<%s>> !%s! ttop10 %s", nick, hand, text);
  tell_top(channel, channel, text, 1, 10, 1);
  return 1;
}

static int pub_ttop20(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  putlog(LOG_CMDS, channel, "<<%s>> !%s! ttop20 %s", nick, hand, text);
  tell_top(channel, channel, text, 11, 20, 1);
  return 1;
}

static void tell_top(char *channel, char *dest, char *type, int from, int to, int today)
{
  char t[50], result[510], tosend[511];
  int found = 0, i, ti;
  struct stats_global *l;
  struct stats_local *ll;
  struct userrec *u;

  Context;
  filt(type);
  if (stat_flood())
    return;
  if (!type[0] || (strlen(type) > 45))
    strcpy(t, "words");
  else
    strcpy(t, type);
  found = typetoi(t);
  if (found == T_ERROR) {
    sprintf(tosend, "PRIVMSG %s :No such stat type.\n", dest);
    dprintf(DP_HELP, "%s", tosend);
    return;
  }
  l = findglobstats(channel);
  if (!l) {
    debug1("StatMod: Can't do topx for %s, no such entry in linked list", channel);
    return;
  }
  sortstats(l, t, today);
  sprintf(result, "%sTop %d(%s):", today ? "Today's " : "", to, t);
  ti = typetoi(t);
  i = 1;
  for (ll = l->local; ll; ll = ll->next) {
    u = get_user_by_handle(userlist, ll->user);
    if (matchattr(u, nostats, channel))
      continue;
    if (!ll->values[today][ti])
      break;
    if ((i >= from) && (i <= to) && ((strlen(result) + strlen(ll->user) + 10) < 450)) {
      sprintf(result, "%s %d. %s(%d)", result, i, ll->user, ll->values[today][ti]);
    }
    i++;
  }
  sprintf(tosend, "PRIVMSG %s :%s\n", dest, result);
  dprintf(DP_HELP, "%s", tosend);
  Context;
}

static int pub_place(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  Context;
  putlog(LOG_CMDS, channel, "<<%s>> !%s! place %s", nick, hand, text);
  tell_place(nick, channel, hand, channel, text, S_TOTAL);
  Context;
  return 1;
}

static int pub_tplace(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  Context;
  putlog(LOG_CMDS, channel, "<<%s>> !%s! tplace %s", nick, hand, text);
  tell_place(nick, channel, hand, channel, text, S_TODAY);
  Context;
  return 1;
}

static void tell_place(char *nick, char *dest, char *hand, char *channel, char *text, int today)
{
  globstats *gs;
  locstats *ls;
  char who[20], tosend[512], type[20];
  int place = 0;
  struct userrec *u;

  Context;
  filt(text);
  if (stat_flood())
    return;
  if ((text[0] == 0) || (strlen(text) > 19)) {
    strcpy(who, hand);
    strcpy(type, "words");
  } else if (typetoi(text) != T_ERROR) {
    strcpy(who, hand);
    strcpy(type, text);
  } else {
    strcpy(who, text);
    strcpy(type, "words");
  }
  if (who[0] == '*') {
    sprintf(tosend, "PRIVMSG %s :%s, sorry, I don't have any stats about you\n",
            dest, nick);
    dprintf(DP_HELP, "%s", tosend);
    return;
  }
  gs = findglobstats(channel);
  if (!gs) {
    debug1("Stats.mod: Couldn't exec !place, I don't have any statistics in %s.", channel);
    return;
  }
  sortstats(gs, "words", today);
  for (ls = gs->local; ls; ls = ls->next) {
    u = get_user_by_handle(userlist, ls->user);
    if (matchattr(u, nostats, channel))
      continue;
    if (!ls->values[today][T_WORDS])
      break;
    place++;
    if (!strcasecmp(who, ls->user))
      break;
  }
  if (!ls) {
    sprintf(tosend, "PRIVMSG %s :%s, sorry, I don't have any stats about %s in %s%s\n",
            dest, nick, who, channel, today ? " today" : "");
    dprintf(DP_HELP, "%s", tosend);
  } else {
    sprintf(tosend, "PRIVMSG %s :%s is nr %d on the list of %stop-talkers.\n", dest,
            who, place, today ? "today's " : "");
    dprintf(DP_HELP, "%s", tosend);
  }
  Context;
}

static int pub_stat(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  Context;
  putlog(LOG_CMDS, channel, "<<%s>> !%s! stat %s", nick, hand, text);
  tell_stat(nick, channel, hand, channel, text, S_TOTAL);
  Context;
  return 1;
}

static int pub_tstat(char *nick, char *host, char *hand,
        char *channel, char *text)
{
  Context;
  putlog(LOG_CMDS, channel, "<<%s>> !%s! tstat %s", nick, hand, text);
  tell_stat(nick, channel, hand, channel, text, S_TODAY);
  Context;
  return 1;
}

static void tell_stat(char *nick, char *dest, char *hand, char *channel, char *text, int today)
{
  locstats *ls;
  char who[20], tosend[512];

  Context;
  filt(text);
  if (stat_flood())
    return;
  if ((text[0] == 0) || (strlen(text) > 19))
    strcpy(who, hand);
  else
    strcpy(who, text);
  if (who[0] == '*') {
    sprintf(tosend, "PRIVMSG %s :%s, sorry, I don't have any stats about you\n",
            dest, nick);
    dprintf(DP_HELP, "%s", tosend);
    return;
  }
  ls = findlocstats(channel, who);
  if (!ls) {
    sprintf(tosend, "PRIVMSG %s :%s, sorry, I don't have any stats about %s in %s\n",
            dest, nick, who, channel);
    dprintf(DP_HELP, "%s", tosend);
  } else {
    sprintf(tosend, "PRIVMSG %s :%s: %d words, %d letters, %d smileys, %s spent on %s\n", dest, who, ls->values[today][T_WORDS],
	    ls->values[today][T_LETTERS], ls->values[today][T_SMILEYS],
	    duration(ls->values[today][T_MINUTES] * 60), channel);
    dprintf(DP_HELP, "%s", tosend);
  }
  Context;
}

static int stat_flood()
{
  if (!maxstat_thr || !maxstat_time)
    return 0;
  if ((now - mstat_time) > maxstat_time) {
    mstat_time = now;
    mstat_thr = 0;
  }
  mstat_thr++;
  if (mstat_thr > maxstat_thr)
    return 1;
  return 0;
}

static cmd_t stats_pub[] =
{
  {"!top10", "", pub_top10, 0},
  {"!ttop10", "", pub_ttop10, 0},
  {"!top20", "", pub_top20, 0},
  {"!ttop20", "", pub_ttop20, 0},
  {"!place", "", pub_place, 0},
  {"!tplace", "", pub_tplace, 0},
  {"!stat", "", pub_stat, 0},
  {"!tstat", "", pub_tstat, 0},
  {0, 0, 0, 0}
};
