/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static int sensor_msgm(char *nick, char *uhost, char *hand, char *chan, char *rest)
{
  if (hand[0] == '*')
    return 0;
  incrstats(hand, chan, T_WORDS, countwords(rest), 0);
  incrstats(hand, chan, T_LETTERS, strlen(rest), 0);
  incrstats(hand, chan, T_LINES, 1, 0);
  incrstats(hand, chan, T_SMILEYS, countsmileys(rest), 0);
  return 0;
}

static void sensor_minutely()
{
  struct chanset_t *chan;
  memberlist *m;

  Context;
  for (chan = chanset; chan; chan = chan->next) {
    if (chan->channel.members > 0) {
      for (m = chan->channel.member; m; m = m->next) {
#ifndef OLDBOT
        if (m->user != NULL)
          incrstats(m->user->handle, chan->dname, T_MINUTES, 1, 0);
        else
          stats_autoadd(m, chan->dname);
#else
        if (m->user != NULL)
          incrstats(m->user->handle, chan->name, T_MINUTES, 1, 0);
        else
          stats_autoadd(m, chan->name);
#endif
      }
    }
  }
  Context;
}

static void sensor_peak(char *channel)
{
  struct chanset_t *chan;
  memberlist *m;
  globstats *gs;
  int users = 0;

  gs = findglobstats(channel);
  if (!gs)
    return;
#ifndef OLDBOT
  chan = findchan_by_dname(channel);
#else
  chan = findchan(channel);
#endif
  if (!chan)
    return;
  if (chan->channel.members > 0) {
    for (m = chan->channel.member; m; m = m->next) {
      if (!m->nick[0])
        continue;
      if (m->user) {
        if (matchattr(m->user, nopeak, channel))
          continue;
      }
      users++;
    }
  }
  if (users > gs->peak[S_TOTAL]) {
    gs->peak[S_TOTAL] = users;
    putlog(LOG_MISC, "*", "New user peak in %s: %d.", channel, users);
  }
  if (users > gs->peak[S_TODAY])
    gs->peak[S_TODAY] = users;
}

static int sensor_topc(char *nick, char *uhost, char *hand, char *chan, char *topic)
{
  Context;
  if (hand[0] == '*')
    return 0;
  incrstats(hand, chan, T_TOPICS, 1, 0);
  Context;
  return 0;
}

static int sensor_action(char *nick, char *uhost, char *hand, char *chan, char *key, char *rest)
{
  Context;
  if (!strchr(CHANMETA, chan[0]))
    return 0;
  if (hand[0] == '*')
    return 0;
  incrstats(hand, chan, T_ACTIONS, 1, 0);
  sensor_msgm(nick, uhost, hand, chan, rest);
  return 0;
}

static int sensor_kick(char *nick, char *uhost, char *hand, char *chan, char *victim, char *reason)
{
  Context;
  if (hand[0] == '*')
    return 0;
  incrstats(hand, chan, T_KICKS, 1, 0);
  Context;
  return 0;
}

static int sensor_mode(char *nick, char *uhost, char *hand, char *chan, char *mode, char *victim)
{
  Context;
  if (hand[0] == '*')
    return 0;
  incrstats(hand, chan, T_MODES, 1, 0);
  if ((mode[1] == 'b') && (mode[0] == '+'))
    incrstats(hand, chan, T_BANS, 1, 0);
  Context;
  return 0;
}

static int sensor_nick(char *nick, char *uhost, char *hand, char *chan, char *newnick)
{
  if (hand[0] == '*')
    return 0;
  incrstats(hand, chan, T_NICKS, 1, 0);
  return 0;
}

static int sensor_join(char *nick, char *uhost, char *hand, char *chan)
{
  sensor_peak(chan);
  if (hand[0] == '*')
    return 0;
  incrstats(hand, chan, T_JOINS, 1, 0);
  return 0;
}

static cmd_t stats_pubm[] =
{
  {"*", "", (Function) sensor_msgm, "stat"},
  {0, 0, 0, 0}
};

static cmd_t stats_topc[] =
{
  {"*", "", (Function) sensor_topc, "stat"},
  {0, 0, 0, 0}
};

static cmd_t stats_ctcp[] =
{
  {"ACTION", "", (Function) sensor_action, "stat"},
  {0, 0, 0, 0}
};

static cmd_t stats_kick[] =
{
  {"*", "", (Function) sensor_kick, "stat"},
  {0, 0, 0, 0}
};

static cmd_t stats_mode[] =
{
  {"*", "", (Function) sensor_mode, "stat"},
  {0, 0, 0, 0}
};

static cmd_t stats_nick[] =
{
  {"*", "", (Function) sensor_nick, "stat"},
  {0, 0, 0, 0}
};

static cmd_t stats_join[] =
{
  {"*", "", (Function) sensor_join, "stat"},
  {0, 0, 0, 0}
};
