/*
*   Statistics module for eggdrop 1.4+
*     by G`Quann
*/

/* 
 * stats.c   - this is never gonna work... (Oct 99)
*/

/*
*  oops... seems that it works :) (Dec 99)
*/

/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#define MAKING_STATS
#define MODULE_NAME "stats"
#define MODULE_VERSION "1.1.9"
#include "../module.h"
#include "../irc.mod/irc.h"
#include "../server.mod/server.h"
#include "../channels.mod/channels.h"
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h> /* for time_t */

#undef global
static Function *global = NULL, *irc_funcs = NULL, *server_funcs = NULL, *channels_funcs = NULL;

#ifndef EGG_VERSION
#define EGG_VERSION 10400
#endif

#define EGG_IS(ver)		((ver) >= EGG_VERSION)

#if EGG_IS(10500)
#define OLDBOT 1
#endif

#ifndef Context
#define Context context
#endif

#ifndef findchan_by_dname
#define findchan_by_dname findchan
#endif

#include "stats.h"

static struct stats_global *sdata = NULL;

static char statsfile[121] = "statsmod.dat";
static char webstats[510] = "words letters lines actions smileys joins kicks modes topics minutes";
static char graphstats[510] = "words letters lines actions smileys joins kicks modes topics minutes";
static char graphgif[128] = "";
static char graphcolor[20] = "blue";
static char webdir[256] = "../public_html";
static char webfiles[256] = "";
static char webfile_suffix[20] = ".html";
static char bodytag[512] = "<BODY BGCOLOR=#000000 TEXT=#1A9DFF LINK=#00D993 VLINK=#71C2FF ALINK=#2FFFBB>";
static char webchans[256] = "";
static char smileys[128] = ":-) :) ;) ;-) ^_^ :-( :( ;( ;-( :-/ :/ ;/ ;-/ :-D :-P :P =) ;D";
static char badflags[20] = "ofvb|ofv";
static char nostats[20] = "b|-";
static char nopeak[20] = "b|-";
static char network[41] = "unknown-net";
static char stats_loglevel[20] = "1";
static char livestats_log[121];
static int statsfilemode = 0600;
static int webupdate = 15;
static int webnr = 15;
static int graphnr = 15;
static int stats_save_time = 10;
static int autoadd = -1;
static int stat_expire_user = 0;
static int write_today = 1;
static int maxstat_thr = 0;
static int maxstat_time = 0;
static int mstat_thr = 0;
static time_t mstat_time = 0;
static int livestats_timeout;
static int offset = 0;

#include "datahandling.c"
#include "sensors.c"
#include "misc.c"
#include "pubcmds.c"
#include "msgcmds.c"
#include "dcccmds.c"
#include "tclstats.c"
#include "webfiles.c"
#include "user.c"
#include "livestats.c"

static int stats_save_temp = 1;
static int webupdate_temp = 1;

static int stats_expmem()
{
  int size = 0;
  struct stats_global *sl = sdata;

  Context;
  while (sl) {
    size += sizeof(struct stats_global);
    size += strlen(sl->chan) + 1;
    size += localstats_expmem(sl->local);
    sl = sl->next;
  }
  Context;
  return size;
}

static int localstats_expmem(struct stats_local *sl)
{
  int size = 0;
  
  Context;
  while (sl) {
    size += sizeof(struct stats_local);
    size += strlen(sl->user) + 1;
    sl = sl->next;
  }
  Context;
  return size;
}

static void free_stats()
{
  struct stats_global *sl;
  
  Context;
  while (sdata) {
    sl = sdata->next;
    free_localstats(sdata->local);
    nfree(sdata->chan);
    nfree(sdata);
    sdata = sl;
  }
  Context;
  return;
}

static void free_localstats(struct stats_local *sl)
{
  struct stats_local *sll;
  
  Context;
  while (sl) {
    Context;
    sll = sl->next;
    Context;
    nfree(sl->user);
    Context;
    nfree(sl);
    Context;
    sl = sll;
    Context;
  }
  Context;
  return;
}

/* a report on the module status */
static void stats_report(int idx, int details)
{
  int size = 0;

  Context;
  if (details) {
    size = stats_expmem();
    dprintf(idx, "    using %d bytes\n", size);
  }
}

static void stats_minutely ()
{
  sensor_minutely();
  if (stats_save_temp >= stats_save_time) {
    write_stats();
    stats_save_temp = 1;
  } else
    stats_save_temp++;
  if ((webupdate_temp >= webupdate) && (webupdate > 0)) {
    webupdate_temp = 1;
    writewebstats(webchans);
  } else
    webupdate_temp++;
}

static void stats_daily ()
{
  Context;
  deloldstatusers();
  purgestats();
  reset_tstats();
  Context;
}

static tcl_ints my_tcl_ints[] =
{
  {"save-stats", &stats_save_time, 0},
  {"webupdate", &webupdate, 0},
  {"webnr", &webnr, 0},
  {"graphnr", &graphnr, 0},
  {"autoadd", &autoadd, 0},
  {"expire-users", &stat_expire_user, 0},
  {"write-today", &write_today, 0},
  {"stats-file-mode", &statsfilemode, 0},
  {"livestats-timeout", &livestats_timeout, 0},
  {"offset", &offset, 0},
  {0, 0, 0}
};

static tcl_strings my_tcl_strings[] =
{
  {"statsfile", statsfile, 121, 0},
  {"webstats", webstats, 510, 0},
  {"webdir", webdir, 256, 0},
  {"webfiles", webfiles, 256, 0},
  {"webfile-suffix", webfile_suffix, 20, 0},
  {"bodytag", bodytag, 512, 0},
  {"webchans", webchans, 256, 0},
  {"smileys", smileys, 128, 0},
  {"graphstats", graphstats, 510, 0},
  {"graphgif", graphgif, 128, 0},
  {"graphcolor", graphcolor, 20, 0},
  {"anti-autoadd-flags", badflags, 20, 0},
  {"anti-stats-flag", nostats, 20, 0},
  {"anti-peak-flag", nopeak, 20, 0},
  {"network", network, 40, 0},
  {"stats-loglevel", stats_loglevel, 20, 0},
  {"livestats-log", livestats_log, 121, 0},
  {0, 0, 0, 0}
};

static tcl_coups my_tcl_coups[] =
{
  {"max-stat-cmds", &maxstat_thr, &maxstat_time},
  {0, 0, 0},
};

static char *stats_close()
{
  Context;
  stop_listen_livestats();
  write_stats();
  free_stats();
  rem_builtins(H_dcc, mydcc);
  rem_builtins(H_pubm, stats_pubm);
  rem_builtins(H_pub, stats_pub);
  rem_builtins(H_msg, stats_msg);
  rem_builtins(H_topc, stats_topc);
  rem_builtins(H_ctcp, stats_ctcp);
  rem_builtins(H_kick, stats_kick);
  rem_builtins(H_mode, stats_mode);
  rem_builtins(H_nick, stats_nick);
  rem_builtins(H_join, stats_join);
  rem_builtins(H_nkch, stats_nkch);
  rem_tcl_ints(my_tcl_ints);
  rem_tcl_coups(my_tcl_coups);
  rem_tcl_strings(my_tcl_strings);
  rem_tcl_commands(mytcls);
  rem_help_reference("stats.help");
  del_hook(HOOK_MINUTELY, (Function) stats_minutely);
  del_hook(HOOK_DAILY, (Function) stats_daily);
  module_undepend(MODULE_NAME);
  return NULL;
}

char *stats_start();

static Function stats_table[] =
{
  (Function) stats_start,
  (Function) stats_close,
  (Function) stats_expmem,
  (Function) stats_report,
};

char *stats_start(Function * global_funcs)
{
  global = global_funcs;
  Context;
  module_register(MODULE_NAME, stats_table, 1, 1);
  if (!(irc_funcs = module_depend(MODULE_NAME, "irc", 1, 0)))
    return "You need the irc module to use the stats module.";
  if (!(server_funcs = module_depend(MODULE_NAME, "server", 1, 0)))
    return "You need the server module to use the stats module.";
  if (!(channels_funcs = module_depend(MODULE_NAME, "channels", 1, 0)))
    return "You need the channels module to use the stats module.";
  if (!module_depend(MODULE_NAME, "eggdrop", 105, 0))
    if (!module_depend(MODULE_NAME, "eggdrop", 104, 0))
      return "This module requires eggdrop1.4.0 or later";
#ifndef OLDBOT
  livestats_timeout = 10;
#else
  livestats_timeout = 1;
#endif
  livestats_log[0] = 0;
  add_builtins(H_dcc, mydcc);
  add_builtins(H_pubm, stats_pubm);
  add_builtins(H_pub, stats_pub);
  add_builtins(H_msg, stats_msg);
  add_builtins(H_topc, stats_topc);
  add_builtins(H_ctcp, stats_ctcp);
  add_builtins(H_kick, stats_kick);
  add_builtins(H_mode, stats_mode);
  add_builtins(H_nick, stats_nick);
  add_builtins(H_join, stats_join);
  add_builtins(H_nkch, stats_nkch);
  add_tcl_ints(my_tcl_ints);
  add_tcl_coups(my_tcl_coups);
  add_tcl_strings(my_tcl_strings);
  add_tcl_commands(mytcls);
  add_hook(HOOK_MINUTELY, (Function) stats_minutely);
  add_hook(HOOK_DAILY, (Function) stats_daily);
  add_help_reference("stats.help");
  read_stats();
  putlog(LOG_MISC, "*", "Stats.mod v%s loaded.", MODULE_VERSION);
  return NULL;
}
